/*
 * Copyright 2010 James Pether Sörling
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *	$Id: ItsYourParliamentEuApiImpl.java 5448 2011-04-26 20:15:51Z pether $
 *  $HeadURL: https://cia.svn.sourceforge.net/svnroot/cia/tags/parent-pom-2011.05.31/service.external.itsyourparliament/src/main/java/com/hack23/cia/service/external/itsyourparliament/impl/ItsYourParliamentEuApiImpl.java $
*/
package com.hack23.cia.service.external.itsyourparliament.impl;

import java.math.BigInteger;
import java.util.List;

import javax.xml.bind.JAXBElement;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.oxm.Unmarshaller;
import org.springframework.stereotype.Component;

import com.hack23.cia.model.external.itsyourparliament.countries.impl.CountriesEuElement;
import com.hack23.cia.model.external.itsyourparliament.countries.impl.Countrydetails;
import com.hack23.cia.model.external.itsyourparliament.countryinfo.impl.CountryinfoEuElement;
import com.hack23.cia.model.external.itsyourparliament.mepinfo.impl.MepinfoEuElement;
import com.hack23.cia.model.external.itsyourparliament.voteinfo.impl.VoteinfoEuElement;
import com.hack23.cia.model.external.itsyourparliament.votes.impl.VotesEuData;
import com.hack23.cia.service.external.common.impl.AbstractXmlAgentImpl;
import com.hack23.cia.service.external.itsyourparliament.api.ItsYourParliamentEuApi;

/**
 * The Class ItsYourParliamentEuApiImpl.
 */
@Component
public final class ItsYourParliamentEuApiImpl extends AbstractXmlAgentImpl implements ItsYourParliamentEuApi {

	/** The eu itsyourparliament countries marshaller. */
	@Autowired
	@Qualifier("euItsyourparliamentCountriesMarshaller")
	private Unmarshaller euItsyourparliamentCountriesMarshaller;

	/** The eu itsyourparliament country info marshaller. */
	@Autowired
	@Qualifier("euItsyourparliamentCountryInfoMarshaller")
	private Unmarshaller euItsyourparliamentCountryInfoMarshaller;

	/** The eu itsyourparliament mep info marshaller. */
	@Autowired
	@Qualifier("euItsyourparliamentMepInfoMarshaller")
	private Unmarshaller euItsyourparliamentMepInfoMarshaller;

	/** The eu itsyourparliament vote info marshaller. */
	@Autowired
	@Qualifier("euItsyourparliamentVoteInfoMarshaller")
	private Unmarshaller euItsyourparliamentVoteInfoMarshaller;

	/** The eu itsyourparliament votes marshaller. */
	@Autowired
	@Qualifier("euItsyourparliamentVotesMarshaller")
	private Unmarshaller euItsyourparliamentVotesMarshaller;



	/**
	 * Check access key.
	 *
	 * @param accessKey
	 *            the access key
	 * @return the string
	 */
	private String checkAccessKey(final String accessKey) {
		String result=accessKey;
		if (accessKey == null || accessKey.contains("$")) {
			result = System.getenv()
					.get("ITSYOURPARLIAMENT_EU_API_KEY");
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.itsyourparliament.api.ItsYourParliamentEuApi#getCountries(java.lang.String)
	 */
	@Override
	public List<Countrydetails> getCountries(final String accessKey,final boolean current) throws Exception {
		final List<Countrydetails> list = ((CountriesEuElement) this.unmarshallXml(euItsyourparliamentCountriesMarshaller,"http://www.itsyourparliament.eu/api/"+  getTermModifier(current) +"countries.php?key="+checkAccessKey(accessKey),"http://countries.itsyourparliament.external.model.cia.hack23.com/impl"," & ", " &amp; ")).getCountrydetails();
		for (final Countrydetails countrydetails : list) {
			countrydetails.setCurrentPeriod(current);
			countrydetails.setParliamentPeriod(getTerm(current));
		}
		return list;
	}

	/* (non-Javadoc)
	 * @see com.hack23.cia.service.external.itsyourparliament.api.ItsYourParliamentEuApi#getCountryInfo(java.lang.String, java.lang.String)
	 */
	@Override
	public CountryinfoEuElement getCountryInfo(final BigInteger id, final String accessKey,final boolean current)
			throws Exception {
		final CountryinfoEuElement countryinfoEuElement=  (CountryinfoEuElement) this.unmarshallXml(euItsyourparliamentCountryInfoMarshaller,"http://www.itsyourparliament.eu/api/"+  getTermModifier(current) +"country.php?id=" + id + "&key=" +checkAccessKey(accessKey),"http://countryinfo.itsyourparliament.external.model.cia.hack23.com/impl"," & ", " &amp; ");
		countryinfoEuElement.setCurrentPeriod(current);
		countryinfoEuElement.setParliamentPeriod(getTerm(current));
		return countryinfoEuElement;
	}


	/* (non-Javadoc)
	 * @see com.hack23.cia.service.impl.control.agent.eu.impl.agents.MepInfoAgent#getMepInfo(java.lang.String)
	 */
	@Override
	public MepinfoEuElement getMepInfo(final String mepId,final String accessKey,final boolean current) throws Exception {
		return ((JAXBElement<MepinfoEuElement>) this.unmarshallXml(euItsyourparliamentMepInfoMarshaller,"http://www.itsyourparliament.eu/api/"+  getTermModifier(current) +"mep.php?id=" +mepId +  "&key="+checkAccessKey(accessKey),"http://mepinfo.itsyourparliament.external.model.cia.hack23.com/impl"," & ", " &amp; ")).getValue();

	}

	/**
	 * Gets the term.
	 *
	 * @param current
	 *            the current
	 * @return the term
	 */
	private String getTerm(final boolean current) {
		if (current) return "2010-2015";
		else return "2004-2009";
	}

	/**
	 * Gets the term modifier.
	 *
	 * @param current
	 *            the current
	 * @return the term modifier
	 */
	private String getTermModifier(final boolean current) {
		if (current) return "";
		else return "0409";
	}



	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.impl.control.agent.sweden.api.PoliticianAgent#
	 * getCurrentList()
	 */
	@Override
	public VoteinfoEuElement getVoteInfo(final String voteId, final String accessKey,final boolean current) throws Exception {
		return (VoteinfoEuElement) this.unmarshallXml(euItsyourparliamentVoteInfoMarshaller,"http://www.itsyourparliament.eu/api/"+  getTermModifier(current) +"vote.php?id=" +voteId +  "&key="+checkAccessKey(accessKey),"http://voteinfo.itsyourparliament.external.model.cia.hack23.com/impl"," & ", " &amp; ");
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * com.hack23.cia.service.impl.control.agent.sweden.api.PoliticianAgent#
	 * getCurrentList()
	 */
	@Override
	public VotesEuData getVotes(final String year,final String accessKey,final boolean current) throws Exception {
		return (VotesEuData) this.unmarshallXml(euItsyourparliamentVotesMarshaller,"http://www.itsyourparliament.eu/api/"+  getTermModifier(current) +"votes.php?y=" + year + "&key=" +checkAccessKey(accessKey),"http://votes.itsyourparliament.external.model.cia.hack23.com/impl"," & ", " &amp; ");
	}

}
